'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var jwt = require('jsonwebtoken');

/**
 * Generate an authentication token which can be sent to the client to
 * verify future requests.
 */
module.exports.generateToken = function generateToken(payload, secret) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var data = Object.assign({
    expiresIn: '30d'
  }, options);
  return _extends({}, data, {
    token: jwt.sign(payload, secret, data)
  });
};

/**
 * Decode a jwt token.
 */
module.exports.decodeToken = function decodeToken(token, secret) {
  return jwt.verify(token, secret);
};

/**
 * Package the authentication.
 */
module.exports.authPackage = function authPackage(payload, secret, options) {
  var _module$exports = module.exports,
      generateToken = _module$exports.generateToken,
      decodeToken = _module$exports.decodeToken;

  var _generateToken = generateToken(payload, secret, options),
      token = _generateToken.token;

  var _decodeToken = decodeToken(token, secret),
      iat = _decodeToken.iat,
      exp = _decodeToken.exp;

  return {
    token: token,
    payload: payload,
    expires: exp,
    iat: iat
  };
};

/**
 * Populate a token found on the request.
 */
module.exports.tokenPopulate = function tokenPopulate(_ref) {
  var Token = _ref.Token,
      secret = _ref.secret;
  var decodeToken = module.exports.decodeToken;

  return function (req, res, next) {
    if (!req.headers || !req.headers.authorization) {
      next();
      return;
    }
    var token = req.headers.authorization;
    if (token === 'null' || token === 'undefined') {
      next();
      return;
    }
    var auth = decodeToken(token, secret);
    Token.findById(auth.id).then(function (issue) {
      if (issue && issue.active) {
        Object.assign(req, { auth: issue });
      }
      next();
    }).catch(next);
  };
};

/**
 * Populate authentication on request if auth found.
 */
module.exports.authPopulate = function authPopulate(_ref2) {
  var User = _ref2.User;

  return function (req, res, next) {
    if (!req.auth) {
      next();
      return;
    }
    if (req.auth && User) {
      User.findById(req.auth.payload.userId).then(function (user) {
        req.user = user;
        next();
      }).catch(next);
    } else {
      next();
    }
  };
};

/**
 * Create and save a token with a user.
 */
module.exports.createUserToken = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        Token = _ref4.Token,
        user = _ref4.user,
        secret = _ref4.secret,
        options = _ref4.options;

    var authPackage, item, pack;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            authPackage = module.exports.authPackage;
            item = new Token({});
            pack = authPackage({
              id: item.id,
              userId: user.id,
              email: user.email
            }, secret, options);
            return _context.abrupt('return', Object.assign(item, pack).save());

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  function createUserToken() {
    return _ref3.apply(this, arguments);
  }

  return createUserToken;
}();